"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrapRollupPlugin = void 0;
const path_1 = __importDefault(require("path"));
const whatwg_url_1 = __importDefault(require("whatwg-url"));
const url_1 = require("url");
const dom5_fork_1 = require("@open-wc/building-utils/dom5-fork");
// @ts-ignore
const building_utils_1 = require("@open-wc/building-utils");
const parse5_1 = require("parse5");
const utils_1 = require("./utils");
const PluginContext_1 = require("./PluginContext");
const NULL_BYTE_PREFIX = '/__es-dev-server-rollup__/';
const NULL_BYTE_PARAM = 'null-byte';
function resolveFilePath(rootDir, path) {
    const fileUrl = new url_1.URL(`.${path}`, `${url_1.pathToFileURL(rootDir)}/`);
    return url_1.fileURLToPath(fileUrl);
}
function wrapRollupPlugin(rollupPlugin, rollupOptions = {}) {
    const transformedFiles = new Set();
    let transformedOptions;
    let fileWatcher;
    let config;
    let rootDir;
    const edsPlugin = {
        serverStart(args) {
            var _a, _b, _c;
            ({ fileWatcher, config } = args);
            ({ rootDir } = config);
            const rollupPluginContext = PluginContext_1.createPluginContext(fileWatcher, config, edsPlugin);
            // call the options and buildStart hooks
            transformedOptions = (_b = (_a = rollupPlugin.options) === null || _a === void 0 ? void 0 : _a.call(rollupPluginContext, rollupOptions)) !== null && _b !== void 0 ? _b : rollupOptions;
            (_c = rollupPlugin.buildStart) === null || _c === void 0 ? void 0 : _c.call(rollupPluginContext, transformedOptions);
        },
        async resolveImport({ source, context }) {
            var _a;
            // if we just transformed this file and the import is an absolute file path
            // we need to rewrite it to a browser path
            const injectedFilePath = transformedFiles.has(context.path) && source.startsWith(rootDir);
            if (!injectedFilePath && !rollupPlugin.resolveId) {
                return;
            }
            if (whatwg_url_1.default.parseURL(source) != null) {
                // don't resolve valid urls
                return source;
            }
            const requestedFile = context.path.endsWith('/')
                ? `${context.path}index.html`
                : context.path;
            const filePath = resolveFilePath(rootDir, requestedFile);
            const rollupPluginContext = PluginContext_1.createPluginContext(fileWatcher, config, edsPlugin, context);
            // if the import was already a fully resolved file path, it was probably injected by a plugin.
            // in that case use that instead of resolving it through a plugin hook. this puts the resolved file
            // path through the regular logic to turn it into a relative browser import
            // otherwise call the resolveID hook on the plugin
            const result = injectedFilePath
                ? source
                : await ((_a = rollupPlugin.resolveId) === null || _a === void 0 ? void 0 : _a.call(rollupPluginContext, source, filePath));
            let resolvedImportFilePath = undefined;
            if (typeof result === 'string') {
                resolvedImportFilePath = result;
            }
            else if (typeof result === 'object' && typeof (result === null || result === void 0 ? void 0 : result.id) === 'string') {
                resolvedImportFilePath = result.id;
            }
            if (!resolvedImportFilePath) {
                return undefined;
            }
            if (resolvedImportFilePath.startsWith(rootDir)) {
                const resolveRelativeTo = path_1.default.extname(filePath)
                    ? path_1.default.dirname(filePath)
                    : filePath;
                const relativeImportFilePath = path_1.default.relative(resolveRelativeTo, resolvedImportFilePath);
                const resolvedImportPath = `${utils_1.toBrowserPath(relativeImportFilePath)}`;
                return resolvedImportPath.startsWith('/') ||
                    resolvedImportPath.startsWith('.')
                    ? resolvedImportPath
                    : `./${resolvedImportPath}`;
            }
            // if the resolved import includes a null byte (\0) there is some special logic
            // these often are not valid file paths, so the browser cannot request them.
            // we rewrite them to a special URL which we deconstruct later when we load the file
            if (resolvedImportFilePath.includes('\0')) {
                return `${NULL_BYTE_PREFIX}?${NULL_BYTE_PARAM}=${encodeURIComponent(resolvedImportFilePath)}`;
            }
            return resolvedImportFilePath;
        },
        async serve(context) {
            var _a;
            if (!rollupPlugin.load) {
                return;
            }
            let filePath;
            if (context.path.startsWith(NULL_BYTE_PREFIX)) {
                // if this was a special URL constructed in resolveImport to handle null bytes,
                // the file path is stored in the search paramter
                filePath = context.URL.searchParams.get(NULL_BYTE_PARAM);
            }
            else {
                filePath = resolveFilePath(rootDir, context.path);
            }
            const rollupPluginContext = PluginContext_1.createPluginContext(fileWatcher, config, edsPlugin, context);
            const result = await ((_a = rollupPlugin.load) === null || _a === void 0 ? void 0 : _a.call(rollupPluginContext, filePath));
            if (typeof result === 'string') {
                return { body: result, type: 'js' };
            }
            if (typeof (result === null || result === void 0 ? void 0 : result.code) === 'string') {
                return { body: result.code, type: 'js' };
            }
            return undefined;
        },
        async transform(context) {
            var _a, _b;
            if (!rollupPlugin.transform) {
                return;
            }
            if (context.response.is('js')) {
                const filePath = resolveFilePath(rootDir, context.path);
                const rollupPluginContext = PluginContext_1.createPluginContext(fileWatcher, config, edsPlugin, context);
                const result = await ((_a = rollupPlugin.transform) === null || _a === void 0 ? void 0 : _a.call(rollupPluginContext, context.body, filePath));
                let transformedCode = undefined;
                if (typeof result === 'string') {
                    transformedCode = result;
                }
                if (typeof result === 'object' && typeof (result === null || result === void 0 ? void 0 : result.code) === 'string') {
                    transformedCode = result.code;
                }
                if (transformedCode) {
                    transformedFiles.add(context.path);
                    return { body: transformedCode };
                }
                return;
            }
            if (context.response.is('html')) {
                const requestedFile = context.path.endsWith('/')
                    ? `${context.path}index.html`
                    : context.path;
                const filePath = resolveFilePath(rootDir, requestedFile);
                let changed = false;
                const documentAst = parse5_1.parse(context.body);
                const scriptNodes = building_utils_1.findJsScripts(documentAst, {
                    jsScripts: true,
                    jsModules: true,
                    inlineJsScripts: true,
                });
                for (const node of scriptNodes) {
                    const code = dom5_fork_1.getTextContent(node);
                    const rollupPluginContext = PluginContext_1.createPluginContext(fileWatcher, config, edsPlugin, context);
                    const result = await ((_b = rollupPlugin.transform) === null || _b === void 0 ? void 0 : _b.call(rollupPluginContext, code, filePath));
                    let transformedCode;
                    if (typeof result === 'string') {
                        transformedCode = result;
                    }
                    else if (typeof result === 'object' &&
                        typeof (result === null || result === void 0 ? void 0 : result.code) === 'string') {
                        transformedCode = result.code;
                    }
                    if (transformedCode) {
                        changed = true;
                        dom5_fork_1.setTextContent(node, transformedCode);
                    }
                }
                if (changed) {
                    transformedFiles.add(context.path);
                    return { body: parse5_1.serialize(documentAst) };
                }
            }
        },
    };
    return edsPlugin;
}
exports.wrapRollupPlugin = wrapRollupPlugin;
